from django.http import HttpRequest, HttpResponse
from django.shortcuts import render
from .context import ModalContext


def update_modal(
    request: HttpRequest,
    template: str,
    context: ModalContext | dict
) -> HttpResponse:

    if isinstance(context, ModalContext):
        context = context.dict()
    res = render(request, template, context)
    res.headers['HX-Retarget'] = f'#{context["modal_id"]}'
    res.headers['HX-Reswap'] = 'outerHTML'
    return res



# Generated by Django 5.1.1 on 2025-01-08 20:04

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accrete', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('model', models.CharField(help_text='Combination of app label and model name seperated by a dot', max_length=255, verbose_name='Model')),
                ('field', models.CharField(max_length=255, verbose_name='Field')),
                ('object_id', models.BigIntegerField(verbose_name='Object ID')),
                ('log_date', models.DateTimeField(auto_now_add=True, verbose_name='Date')),
                ('old_value_type', models.CharField(choices=[('fk', 'Foreign Key'), ('int', 'Integer'), ('float', 'Float'), ('decimal', 'Decimal'), ('bool', 'Boolean'), ('str', 'String')], max_length=100, verbose_name='Old Value Type')),
                ('new_value_type', models.CharField(choices=[('fk', 'Foreign Key'), ('int', 'Integer'), ('float', 'Float'), ('decimal', 'Decimal'), ('bool', 'Boolean'), ('str', 'String')], max_length=100, verbose_name='New Value Type')),
                ('old_value', models.TextField(blank=True, null=True, verbose_name='Old Value')),
                ('new_value', models.TextField(blank=True, null=True, verbose_name='New Value')),
                ('tenant', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='accrete.tenant', verbose_name='Tenant')),
            ],
            options={
                'verbose_name': 'Log',
                'verbose_name_plural': 'Logs',
                'db_table': 'accrete_log',
                'indexes': [models.Index(fields=['model', 'object_id'], name='accrete_log_model_cf888c_idx')],
                'constraints': [models.UniqueConstraint(fields=('model', 'field', 'object_id', 'log_date'), name='unique_group_fields')],
            },
        ),
    ]



# Generated by Django 5.1.4 on 2025-03-06 09:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accrete', '0004_rename_accessgroupmember_memberaccessgrouprel_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='accessgroup',
            name='apply_on',
            field=models.CharField(choices=[('tenant', 'Tenant'), ('member', 'Member')], default='tenant', help_text='', max_length=10, verbose_name='Apply On'),
        ),
        migrations.AlterField(
            model_name='member',
            name='access_groups',
            field=models.ManyToManyField(blank=True, through='accrete.MemberAccessGroupRel', to='accrete.accessgroup'),
        ),
        migrations.AlterField(
            model_name='tenant',
            name='access_groups',
            field=models.ManyToManyField(blank=True, through='accrete.TenantAccessGroupRel', to='accrete.accessgroup'),
        ),
    ]



import logging
import re
from dataclasses import dataclass, field
from django.core import paginator
from django.db.models import QuerySet, Model
from .filter import Filter

_logger = logging.getLogger(__name__)


@dataclass(kw_only=True)
class BaseContext:

    extra: dict = field(default_factory=dict)

    def __post_init__(self):
        for key, value in self.extra.items():
            setattr(self, key, value)

    def dict(self):
        return {
            attr: getattr(self, attr, None) for attr
            in filter(lambda x: not x.startswith('_'), self.__dict__)
        }


@dataclass(kw_only=True)
class Context(BaseContext):

    title: str = ''


@dataclass
class ListContext(Context):

    page: paginator.Page
    queryset: QuerySet
    endless_scroll: bool = True
    detail_enabled: bool = True
    filter: Filter = None
    column_count: int = 1
    column_height: int = 150
    column_height_unit: str = 'px'

    def __post_init__(self):
        super().__post_init__()
        if self.column_count not in range(1, 13):
            _logger.warning(
                'ListContext parameter column_count should be in range 1 - 12'
            )


@dataclass
class ListUpdateContext(BaseContext):

    object: Model
    template: str


@dataclass
class TableContext(Context):

    object_label: str
    fields: list[str]
    page: paginator.Page
    queryset: QuerySet
    footer: dict = field(default_factory=dict)
    endless_scroll: bool = True
    detail_enabled: bool = True
    filter: Filter = None


@dataclass
class TableRowContext(BaseContext):

    object: Model
    fields: list[str]
    queryset: QuerySet
    footer: dict = field(default_factory=dict)


@dataclass
class ModalContext(BaseContext):

    title: str
    modal_id: str
    blocking: bool = False

    def __post_init__(self):
        super().__post_init__()
        self.modal_id = re.sub(r'[^A-Za-z]+','', self.modal_id)



# Generated by Django 5.1.4 on 2025-02-13 18:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='theme',
            field=models.CharField(choices=[('light', 'Light'), ('dark', 'Dark')], default='light', max_length=50, verbose_name='Theme'),
        ),
    ]



from . import widgets
from .filter import Filter
from . context import (
    Context,
    ListContext,
    ListUpdateContext,
    TableContext,
    TableRowContext,
    ModalContext
)
from .utils import update_modal



# Generated by Django 5.1.1 on 2025-01-10 15:20

import django.db.models.deletion
import django.db.models.functions.text
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('log', '0003_alter_log_tenant'),
    ]

    operations = [
        migrations.CreateModel(
            name='LogConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('model', models.CharField(help_text='Combination of app label and model name seperated by a dot', max_length=255, verbose_name='Model')),
                ('ignore_errors', models.BooleanField(default=False, help_text='IF true, exceptions during log creation will be ignored', verbose_name='Ignore Errors')),
            ],
            options={
                'verbose_name': 'Log Configuration',
                'verbose_name_plural': 'Log Configs',
                'db_table': 'accrete_log_config',
                'ordering': [django.db.models.functions.text.Lower('model')],
                'indexes': [models.Index(fields=['model'], name='accrete_log_model_a89071_idx')],
                'constraints': [models.UniqueConstraint(fields=('model',), name='unique_model')],
            },
        ),
        migrations.CreateModel(
            name='LogConfigField',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field_name', models.CharField(help_text='Name of the field to log', max_length=255, verbose_name='Field Name')),
                ('log_config', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='log.logconfig', verbose_name='Log Configuration')),
            ],
            options={
                'verbose_name': 'Log Configuration Field',
                'verbose_name_plural': 'Log Configuration Fields',
                'db_table': 'accrete_log_config_field',
                'ordering': [django.db.models.functions.text.Lower('field_name')],
                'indexes': [models.Index(fields=['log_config'], name='accrete_log_log_con_22f45e_idx')],
                'constraints': [models.UniqueConstraint(fields=('log_config', 'field_name'), name='unique_config_field_name')],
            },
        ),
    ]



from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth import views, update_session_auth_hash
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render, resolve_url
from django.utils.translation import gettext_lazy as _
from django.conf import settings

from accrete.utils import save_form
from accrete.contrib import ui
from .forms import UserForm, ChangePasswordForm, ChangeEmailForm


class LoginView(views.LoginView):

    form_class = AuthenticationForm
    template_name = 'user/login.html'
    redirect_authenticated_user = True

    def form_invalid(self, form):
        user = form.get_user()
        if user is not None and not user.is_active:
            ctx = {'to_confirm': True}
            if self.extra_context:
                self.extra_context.update(ctx)
            else:
                self.extra_context = ctx
        return super().form_invalid(form)


class LogoutView(views.LogoutView):

    def get_success_url(self):
        return resolve_url(settings.LOGIN_URL)


@login_required()
def user_detail(request):
    form = UserForm(
        initial={'language_code': request.user.language_code},
        instance=request.user
    )
    ctx = ui.Context(
        title=_('User Preferences'),
        extra=dict(
            user=request.user,
            form=form
        )
    ).dict()
    if request.method == 'POST':
        form = save_form(UserForm(request.POST, instance=request.user))
        ctx.update(form=form)
        if form.is_saved:
            res = render(request, 'user/user_form.html', ctx)
            res.headers['HX-Refresh'] = 'true'
            return res
    return render(request, 'user/user_form.html', ctx)


@login_required()
def user_change_password(request):
    form = ChangePasswordForm(instance=request.user)
    ctx = ui.ModalContext(
        title=_('Change Password'),
        modal_id='change-password',
        blocking=True,
        extra=dict(
            form=form,
            user=request.user
        )
    ).dict()
    if request.method == 'POST':
        form = save_form(ChangePasswordForm(request.POST, instance=request.user))
        if form.is_saved:
            update_session_auth_hash(request, form.instance)
            return redirect(
                resolve_url('user:detail')
                + f'?{request.GET.urlencode()}'
            )
        ctx.update(form=form)
        return ui.update_modal(request, 'user/change_password.html', ctx)
    return render(request, 'user/change_password.html', ctx)


@login_required()
def user_change_email(request):
    form = ChangeEmailForm(instance=request.user)
    ctx = ui.ModalContext(
        title=_('Change E-Mail'),
        modal_id='change-email',
        blocking=True,
        extra=dict(
            form=form,
            user=request.user
        )
    ).dict()
    if request.method == 'POST':
        form = save_form(ChangeEmailForm(request.POST, instance=request.user))
        if form.is_saved:
            return redirect('user:detail')
        ctx.update(form=form)
        return ui.update_modal(request, 'user/change_email.html', ctx)
    return render(request, 'user/change_email.html', ctx)



# Generated by Django 5.1.1 on 2025-01-10 08:02

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accrete', '0003_remove_member_name'),
        ('log', '0002_log_user_alter_log_new_value_type_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='tenant',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='accrete.tenant', verbose_name='Tenant'),
        ),
    ]



from . import dates
from .forms import (
    save_form,
    save_forms,
    inline_vals_from_post,
    extend_formset
)
from .views import (
    page_from_querystring,
    get_page,
    filter_from_querystring,
    cast_param,
    method_not_allowed,
    render_templates
)
from .models import get_related_model



from django.contrib import admin
from .models import User


class UserAdmin(admin.ModelAdmin):

    model = User
    list_display = (
        'email',
        'username',
        'first_name',
        'last_name'
    )
    search_fields = [
        'email',
        'username',
        'first_name',
        'last_name'
    ]
    list_filter = [
        'is_superuser',
        'is_staff',
        'is_active'
    ]


admin.site.register(User, UserAdmin)
